package ca.uwaterloo.crysp.datacollector;

import java.util.ArrayList;

import android.view.MotionEvent;

public class TouchFeatures {
	/*
	 * Swipe features used by Li et al., NDSS 2014
	 * 0- Direction: pivot for different training models
	 * 1,2- First touch position: x & y co-ords
	 * 3- First touch area: touch area of first touch co-ord
	 * 4- First moving direction: angle between horizontal line and this
	 * 5- Moving distance: sum of distance b/w consecutive points
	 * 6- Duration: time taken to swipe
	 * 7- Average moving direction: Avg val. of moving direction of n-1 pts
	 * 8- Average moving curvature: Avg val. of moving curvature of n-1 pts
	 * 					b/w consecutive pts ABC it is angle ABC 
	 * 9- Average touch area: Avg. of all touch areas
	 * 10- Max area: Index of Max-area pt./Total pts
	 */
	
	
	public static int featureSetSize = 11;
	static double [] fv = new double[featureSetSize];
	
	static ArrayList<TouchPoint> points = new ArrayList<TouchPoint>();

	public static ArrayList<TouchPoint> getTouchPoints() {
		return points;
	}
	
	public static double[] getFV() {
		double [] _fv = new double[featureSetSize];
		for (int i = 0; i < fv.length; i++)
			_fv[i] = fv[i];
		return _fv;
	}
	private static void populateFeatureVector() {
		/*0- Direction: pivot for different training models
		 * code: 0->Up; 1->Down; 2->Left; 3->Right*/
		float xDisp = points.get(0).x - points.get(points.size()-1).x;
		float yDisp = points.get(0).y - points.get(points.size()-1).y;
		if (Math.abs(xDisp) > Math.abs(yDisp))  /*L or R*/
			fv[0] = (xDisp > 0)? 2:3;
		else  /*U or D*/
			fv[0] = (yDisp > 0)? 0:1;
		
		/* 1,2- First touch position: x & y co-ords*/
		fv[1] = points.get(0).x;
		fv[2] = points.get(0).y;
		/* 3- First touch area: touch area of first touch co-ord*/
		fv[3] = points.get(0).area;
		/* 4- First moving direction: angle between horizontal line and this*/
		fv[4] = movingDirection(points.get(0), points.get(1));
		/* 6- Duration: time taken to swipe*/
		fv[6] = points.get(points.size()-1).tstamp - points.get(0).tstamp;
		fv[5] = fv[7] = fv[8] = fv[9] = fv[10] = 0;
		float maxArea = 0;
		for (int i = 0; i < points.size(); i++) {
			if (i < points.size() - 1) { 
				/* 5- Moving distance: sum of distance b/w consecutive points*/
				fv[5] += Math.sqrt(Math.pow(points.get(i+1).y - points.get(i).y, 2)
						+ Math.pow(points.get(i+1).x - points.get(i).x, 2)); 
				/*7- Average moving direction: Avg of moving direction of n-1 pts*/
				fv[7] += movingDirection(points.get(i), points.get(i+1));
			}
			if (i < points.size() - 2)
				/* 8- Average moving curvature: Avg val. of moving curvature of n-1 pts
				 * 	  b/w consecutive pts ABC it is angle ABC*/
				fv[8] += movingCurvature(points.get(i), points.get(i+1), points.get(i+2));
			/* 9- Average touch area: Avg. of all touch areas */
			fv[9] += points.get(i).area;
			/* 10- Max area: Index of Max-area pt./Total pts*/
			if (points.get(i).area >= maxArea) {
				fv[10] = i;
				maxArea = points.get(i).area;
			}
		}
		fv[7] /= points.size() - 1;
		fv[8] /= points.size() - 2;
		fv[9] /= points.size();
		fv[10] /= points.size();
		
		//points.clear();
		/*
		MainActivity.updateView(fv);
		if (MainActivity.state == MainActivity.TRAIN)
			ModelStorage.writeModel(MainActivity.userName, fv);
		else if (MainActivity.state == MainActivity.TEST) {
			MainActivity.clsfr.compare(fv);
			MainActivity.provideEstimates(
					MainActivity.clsfr.estimates[(int)fv[0]]);
			for (int i = 0; i <MainActivity.clsfr.estimates[(int)fv[0]].length; i++)
				Log.e(String.valueOf((int)fv[0]), String.valueOf(MainActivity.clsfr.estimates[(int)fv[0]][i]));
		}*/
	}
	public static boolean procEvent(MotionEvent ev) {
    	switch (ev.getAction()) {
    	case MotionEvent.ACTION_DOWN:  /* primary pointer */
		case MotionEvent.ACTION_POINTER_DOWN: /* any subsequent pointer */
			points.clear();//clear points now; must've been fetched by now
			WebActivity.setSwiping(true);
			return false;
		case MotionEvent.ACTION_MOVE: /* any number of pointers move */
			for (int hIndx = 0; hIndx < ev.getHistorySize(); hIndx++) {
				for (int pIndex = 0; pIndex < ev.getPointerCount(); 
						pIndex++) {
					TouchPoint tp = new TouchPoint();
					tp.x = ev.getHistoricalX(pIndex, hIndx);
					tp.y = ev.getHistoricalY(pIndex, hIndx);
					tp.area = ev.getHistoricalSize(pIndex, hIndx);
					tp.tstamp = ev.getHistoricalEventTime(hIndx);
					tp.pressure = ev.getHistoricalPressure(hIndx);
					points.add(tp);
				}
			}
			for (int pIndex = 0; pIndex < ev.getPointerCount(); 
					pIndex++) {
				TouchPoint tp = new TouchPoint();
				tp.x = ev.getX(pIndex);
				tp.y = ev.getY(pIndex);
				tp.area = ev.getSize(pIndex);
				tp.tstamp = ev.getEventTime();
				tp.pressure = ev.getPressure();
				points.add(tp);
			}
			return false;
		case MotionEvent.ACTION_POINTER_UP: /* all pointers are up */
		case MotionEvent.ACTION_UP: 
		case MotionEvent.ACTION_CANCEL:
			WebActivity.setSwiping(false);
			if (points.size() > 5) {
				populateFeatureVector();
				return true;
			}
			//points.clear();
			return false;
    	}
    	return false;
	}
	private static double movingDirection(TouchPoint pt1, TouchPoint pt2) {
		double angleSelf = Math.atan2(pt1.y - pt2.y, pt1.x - pt2.x);
		double angleHorizontal = Math.atan2(pt1.y , pt1.x - pt2.x);
		return angleSelf - angleHorizontal;
	}
	
	private static double movingCurvature(TouchPoint pt1, TouchPoint pt2, 
			TouchPoint pt3) {
		double anglePt1Pt2 = Math.atan2(pt1.y - pt2.y, pt1.x - pt2.x);
		double anglePt2Pt3 = Math.atan2(pt2.y - pt3.y, pt2.x - pt3.x);
		return anglePt1Pt2 - anglePt2Pt3; 
	}
}
